.\"
.\" ${TROFF} ${MSMACROS} thisfile | ${PRINTER}
.\"
.\" revision date - change whenever this file is edited
.ds RD 29 May 1992
.nr PO 1.2i	\" page offset 1.2 inches
.nr PD .7v	\" inter-paragraph distance
.\"
.EH 'troffcvt Action Reference'- % -''
.OH ''- % -'troffcvt Action Reference'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.\" I - italic font (taken from -ms and changed)
.de I
.nr PQ \\n(.f
.if t \&\\$3\\f2\\$1\\fP\&\\$2
.if n .if \\n(.$=1 \&\\$1
.if n .if \\n(.$>1 \&\\$1\c
.if n .if \\n(.$>1 \&\\$2
..
.\" B - bold font (taken from -ms and changed)
.de B
.nr PQ \\n(.f
.if \\n(.$=0 .ft 3
.if \\n(.$>0 \&\\$3\\f3\\$1\\fP\&\\$2
..
.de St	\" troffcvt special text
\\&\\$3\fB@\\$1\fR\\$2
..
.de Cl	\" troffcvt control
\\&\\$3\fB\e\\$1\fR\\$2
..
.de Rq	\" troff request
\\&\\$3\fB\.\\$1\fR\\$2
..
.de Es	\" troff escape
\\&\\$3\fB\e\\$1\fR\\$2
..
.de Ac	\" action
.LP
.ta \n(LLuR
\(bu \\$1	(\fB\\$2\fR)
.br
..
.TL
troffcvt Action Reference
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.NH
Introduction
.LP
This document describes
the actions that can be used in action files,
and describes how they work.
This includes argument parsing, transmission of argument values, and
action syntax.
.NH
General Syntax
.LP
The
.I actions
file is a plain text file.
Blank lines are ignored, as are lines beginning with ``#'' in column 1, which
are taken as comments.
Other lines should begin with
.B imm
or
.B req
and have the following syntax:
.LP
.DS
imm \fIimmediate-actions\fR
req \fIrequest-name parsing-actions\fR eol \fInon-parsing-actions\fR
.DE
.LP
.B imm
and
.B req
lines may be continued onto the next line with a ``\e'' as the last
character.
.LP
An
.B imm
line specifies actions to be executed as soon as they are read from the
.I actions
file.
A
.B req
line describes what
.I troffcvt
should do when a given request (or macro) occurs in the input.
.I request-name
is the name of the request (without the leading period).
.I parsing-actions
is either empty or a list of actions to execute to parse the request arguments.
.B eol
is mandatory and causes
.I troffcvt
to skip to the end of the request line.
.I non-parsing-actions
is either empty of a list of actions to execute after the request arguments
have been parsed.
In the section below that details what each action does, it's indicated
whether a given action can be used on
.B imm
lines, and the parsing or non-parsing section of
.B req
lines.
.NH
Argument Processing
.NH 2
Argument Collection
.LP
Several actions are available to parse various kinds
of request line arguments,
e.g., numbers, macro arguments, single characters.
Most parsing routines read arguments from the request line, store them
in the request argument vector
.I rargv [\|],
and bump the request argument count
.I rargc .
Arguments stored this way are available to actions occurring later in
the same action list, as
.B $1 ,
.B $2 ,
etc.
.LP
Some parsing routines do not store the arguments in
.I rargv [\|]
but instead act on them directly.
For example,
.B parse-tab-stops
reads the tab stops into an array and uses it to set the current stops.
.B parse-condition
reads and tests the condition, then executes or skips the rest of
the conditional request line.
.NH 2
Argument Transmission \*- General
.LP
When an argument is passed to an action, it is interpreted to perform
request argument substitution and some escape processing.
.LP
.DS
.ta 1i
\fB$\fIn\fR	becomes the \fIn\fR-th request argument, empty if unavailable
\fB$$\fR	becomes the number of arguments parsed from request line
\fB\e\e\fR	becomes \fB\e\fR
\fB\en\fR	becomes a linefeed
\fB\et\fR	becomes a tab
\fB\e\fIX\fR	becomes \fIX\fR, any character not listed above
.DE
An empty string is substituted for a request argument if
.I n
is not a digit, there is no such argument, the argument was empty or
the action is in an immediate action list.
.LP
Quotes may be used around action arguments to include whitespace.
.LP
.B Example:
If there are two request arguments available, then the following would
have the effect of defining a string if the first argument isn't empty:
.LP
.DS
push-string ".if!\'$1\'\' .ds $1 \e"$2"
.DE
.NH 2
Argument Transmission \*- Numeric Arguments
.LP
.LP
The
.B parse-num
action is used to parse a number from the request line.
It takes one argument, a scaling indicator used to scale numbers
in request argument expressions that don't have explicit scaling indicators.
For example, the
.Rq sp
request can be specified this way:
.LP
.DS
req sp parse-num v eol break space $1
.DE
.LP
In this case
.B parse-num
uses
.B v 's
as the default scaling.
Then the arguments to the requests
.LP
.DS
\&.sp 1
\&.sp 1i
\&.sp 1i+1
.DE
.LP
will be interpreted as ``1v'', ``1i'' and ``1i+1v'', respectively.
(The ``i'', where present, overrides the default of
.B v ).
.LP
In each case, after
.B parse-num
finishes evaluating the expression, the result is converted into basic
units, and that is what is stored in the request argument vector.
If the resolution is 432 units/inch and a
.B v
is 72 units, the arguments for the three requests above would be stored
as the strings ``72'', ``432'' and ``504''.
In other words, the value stored may not look much like the value on
the request line.
.LP
Allowable scaling indicators are
.B i
(inches),
.B c
(centimeters),
.B P
(picas),
.B m
(ems),
.B n
(ens),
.B p
(points),
.B v
(vertical spacing units),
.B u
(basic units) and
.B x
(ignore scaling).
.LP
The
.B x
specifier is special; it means there is no default, numbers are interpreted
without scaling, and the result stored in the request argument vector is
.I not
converted to basic units.
This is appropriate for requests such as
.Rq ce
or
.Rq ls ,
which may be specified like so:
.LP
.DS
req ce parse-num x eol center $1
req ls parse-num x eol line-spacing $1
.DE
.LP
If a request argument is missing from the request line, it is stored in
.I rargv [\|]
as the empty string.
Functions for actions which allow arguments to be missing (empty) must be
written to test for that case.
.LP
Actions that take numeric parameters usually will expect
that the number be given in basic units, unless an explicit scaling
indicator is appended.
The reason for this is because numeric arguments to actions are generally
obtained from parsing actions
earlier in the action list, and those
will usually have been converted to basic units.
This means that even for actions used in
.B imm
lines, the same convention must be followed.
Thus, the two lines following have very different results (the second
is correct if you want the vertical spacing set to 12 points):
.LP
.DS
imm space 12
imm space 12p
.DE
.NH 2
Argument Transmission \*- Relative-Change Requests
.LP
Some requests allow parameter values to be changed either to an
absolute value, or relatively by a specified amount.
Examples:
.LP
.DS
.ta 1i
\&.ps 3	set point size to 3 points
\&.ps +3	... to 3 points greater than current value
\&.nr x 4	define number register with initial value 4
\&.nr x +4	...with initial value 4 greater than current value.
.DE
.LP
This kind of request is handled with the
.B parse-absrel-num
action.
.B parse-absrel-num
is like
.B parse-num
in that it takes a default scaling indicator, but it also takes a second
argument which specifies the current value of the relevant parameter.
Since you can't specify the current value literally in an action file
(it may change!), you can refer to the parameter symbolically.
Those which may be specified this way are
.B point-size ,
.B spacing ,
.B line-spacing ,
.B indent ,
.B line-length ,
.B page-length ,
.B title-length
and
.B offset .
.LP
.B Example:
the current indent may be set absolutely, or relative to the current
value, so it may be specified like this:
.LP
.DS
req in parse-absrel-num m indent eol break indent $1
.DE
.LP
If the argument on an
.Rq in
request begins with a sign,
the sign is stripped, the rest of the argument is evaluated
to produce a number, and the result is the current indent modified up
or down by that number.
(The result does
.I not
modify the current indent; it is simply stored in
.I rargv [\|].)
If there is no initial sign, the request argument is an absolute number
and the parse result is simply the value of the argument (the current
indent value is ignored).
In the example above, the result goes into
.B $1 .
The second reference to
.B indent
is as an action, used to actually set the indentation to the value of
.B $1 .
.LP
As a special-case hack, if the current-value argument to
.B parse-absrel-num
begins with a backslash, the rest of the argument is taken as
the name of a number register, and the value of the register
is used.
This is used in relation to the
.Rq nr
request, which allows a register to be redefined relative to
its current value.
.Rq nr
can be specified in an action file as:
.LP
.DS
req nr parse-name parse-absrel-num u \e\e$1 parse-num x eol define-register $1 $2 $3
.DE
.LP
It is necessary that
.B parse-num
and
.B parse-absrel-num
both exist,
because an expression such as ``-10+10'' means ``0'' as an absolute number,
but ``decrease by 20'' as a relative number.
.NH
Action Descriptions
.LP
Each action is discussed below, along with a key as to where it can be
used:
.DS
.ta 1i
req-p	allowable in argument parsing section of \fBreq\fR lines
req-np	allowable in non-parsing section of \fBreq\fR lines
imm	allowable in \fBimm\fR lines
.DE
.LP
All arguments listed for an action
.I must
be present whenever the action is used, although an action can often be
the empty string.
Thus
.B "escape-char @"
sets the escape character to ``@'' while \fBescape-char ""\fR
restores the default escape character.
.LP
Actions that normally generate output do not do so when the action file
is being read, because
.I troffcvt
turns off output until it knows the complete initial layout (i.e., until
after the action file has been read).
However, even though
.B imm
lines don't generate output immediately, they do take effect immediately.
.Ac "\fBabort\fR \fIstring\fR" "req-np/imm"
.IP
Aborts
.I troffcvt
after printing
.I string
on
.I stderr .
If
.I string
is empty, ``User abort'' is printed instead.
.Ac "\fBadjust\fR \fIc\fR" "req-np/imm"
.IP
Turns on output line adjusting, with mode
.I c ,
which can be
.B l
(left),
.B c
(center),
.B r
(right),
.B b
or
.B n
(both),
or a number previously obtained from the
.B \.j
register.
An adjustment indicator is written to the output if the request changes
the current adjustment and filling is on.
If
.I c
is missing, adjustment is turned on with whatever the current mode is.
.Ac "\fBappend-diversion\fR \fIxx\fR" req-np
.IP
Like
.B begin-diversion
but writes
.Cl append-diversion
.I xx
to the output instead.
.Ac "\fBappend-macro\fR \fIxx yy\fR" req-np
.IP
Like
.B define-macro
but appends to an existing macro definition.
.Ac "\fBappend-string\fR \fIxx value\fR" "req-np/imm"
.IP
Like
.B define-string
but appends to an existing string.
.Ac "\fBbegin-diversion\fR \fIxx\fR" req-np
.IP
Writes
.Cl begin-diversion
.I xx
to the output.
When a matching
.Rq di
or
.Rq da
is seen,
.Cl end-diversion
.I xx
is written.
.Ac "\fBbegin-page\fR \fIN\fR" "req-np/imm"
.IP
If
.I N
is empty, writes
.Cl begin-page .
Otherwise, writes
.Cl begin-page
.I N
and sets the current page number and register
.B %
to
.I N .
.Ac "\fBbreak\fR" "req-np/imm"
.IP
Causes a break (writes a
.Cl break
line) unless the no-break control character was given
on the request or unless the current output line doesn't have anything on it.
None of the action functions do a break except that for
.B break
itself, so it
should be specified in the action list for any request that normally
causes a break.
The reason for this is to make sure complete control over when a break occurs
is in the action file, not inside
.I troffcvt .
This is more work for the action file writer, but provides more
flexibility.
.IP
For instance,
.Rq sp
can be specified as ``req sp parse-num v eol break space $1''.
The function associated with
.B space
doesn't do any break itself, so the
.B break
is necessary.
.Ac "\fBcenter\fR \fIN\fR" "req-np/imm"
.IP
Turns on centering and writes
.Cl center
to the output.
Centering is turned off after
.I N
lines
of text input
(1 line if
.I N
is empty).
.Ac "\fBconstant-width\fR \fIF N M\fR" "req-np/imm"
.IP
Umm...
.Ac "\fBcontinuous-underline\fR \fIN\fR" "req-np/imm"
.IP
Turns on continuous underlining and writes
.Cl cunderline
to the output.
Continuous underlining is turned off after
.I N
lines
of text input
(1 line if
.I N
is empty).
.Ac "\fBdebug-flag\fR \fIN\fR" req-np/req-p/imm
.IP
Turns on debugging flag bit
.I N .
The allowable values are listed in
.I troffcvt.h
and are not of general interest.
If
.I N
is missing, all flags are turned on.
If
.I N
is negative, all flags are turned off.
.Ac "\fBdefine-macro\fR \fIxx yy\fR" req-np
.IP
Defines a macro named
.I xx .
Reads input in copy mode until a line of the form \fB\.\fIyy\fR is found,
at which point \fB\.\fIyy\fR is invoked.
If
.I yy
is missing, a line of the form
.B \.\.
terminates the definition.
.Ac "\fBdefine-register\fR \fIxx init incr\fR" "req-np/imm"
.IP
Defines number register
.I xx
with initial value
.I init
and increment value
.I incr .
.Ac "\fBdefine-string\fR \fIxx value\fR" "req-np/imm"
.IP
Defines string
.I xx
with value
.I value .
.Ac "\fBdump-macro\fR \fIxx\fR" "req-np/req-p/imm"
.IP
Dumps body of macro
.I xx
to
.I stderr .
This is for debugging.
.Ac "\fBecho\fR \fIstring\fR" "req-p/req-np/imm"
.IP
Writes
.I string
to
.I stderr .
.Ac "\fBembolden\fR \fIS F N\fR" "req-np/imm"
.IP
Emboldens font
.I F
by smearing it
.I N
units.
If
.I S
is not empty, embolden the special font by
.I N
units when the current font is
.I F .
.Ac "\fBend-input\fR" req-np
.IP
Causes input routines to return EOF so that troffcvt will finish processing
immediately.
.Ac "\fBend-macro\fR \fIxx\fR" "req-np/imm"
.IP
Causes macro
.I xx
to be invoked when all other input has been processed.
.Ac "\fBenvironment\fR \fIn\fR" "req-np/imm"
.IP
Enters environment
.I n
(\fIn\fR is 0, 1 or 2)
or resume previous environment if
the argument is empty.
.Ac "\fBeol\fR" "req-p/req-np"
.IP
Skips to the end of the current input line.
.Ac "\fBfill\fR" "req-np/imm"
.IP
Turns on fill mode.
.Ac "\fBfont\fR \fIF\fR" "req-np/imm"
.IP
Switches to font
.I F ,
writing
.Cl font
.I F
to the output if this causes a font change.
.Ac "\fBfont-position\fR \fIN F\fR" "req-np/imm"
.IP
Associates font
.I F
with position
.I N .
.Ac "\fBhyphenate\fR \fImode\fR" "req-np/imm"
.IP
Sets the hyphenation mode, which should be zero to turn off hyphenation,
or a positive value as described in the
.I troff
manual.
.Ac "\fBhyphen-char\fR \fIc\fR" "req-np/imm"
.IP
Sets the hyphenation character to
.I c .
.Ac "\fBignore\fR \fIyy\fR" "req-np/imm"
.IP
Input is read in copy mode but otherwise ignored until a line of the form
\fB\.\fIyy\fR is found, at which time
.Rq \fIyy\fR
is invoked.
If
.I yy
is missing, a line of the form
.B \.\.
terminates the
.B ignore
action.
.Ac "\fBindent\fR \fIN\fR" "req-np/imm"
.IP
Sets indent to
.I N .
.Ac "\fBline-length\fR \fIN\fR" "req-np/imm"
.IP
Sets line length to
.I N .
.Ac "\fBline-spacing\fR \fIN\fR" "req-np/imm"
.IP
Sets line spacing to
.I N .
.Ac "\fBno-space\fR \fIc\fR" "req-np/imm"
.IP
Turns no-space mode on if the argument is ``y'', off if it's ``n''.
.Ac "\fBnoadjust\fR" "req-np/imm"
.IP
Turns off output line adjusting.
.Ac "\fBnoescape\fR" "req-np/imm"
.IP
Turns off escape processing.
.Ac "\fBnofill\fR" "req-np/imm"
.IP
Turns on no-fill mode.
.Ac "\fBoffset\fR \fIN\fR" "req-np/imm"
.IP
Sets offset to
.I N .
.Ac "\fBoutput-control\fR \fIstring\fR" "req-np/imm"
.IP
Writes
.I string
directly to the output, preceded by a
.B \e .
.I string
should not contain escaped or special characters.
.Ac "\fBoutput-special\fR \fIstring\fR" "req-np/imm"
.IP
Writes
.I string
directly to the output, preceded by a
.B @ .
.I string
should not contain escaped or special characters.
.Ac "\fBoutput-text\fR \fIstring\fR" "req-np/imm"
.IP
Writes
.I string
directly to the output.
.I string
should not contain escaped or special characters.
.Ac "\fBpage-length\fR \fIN\fR" "req-np/imm"
.IP
Sets page length to
.I N .
.Ac "\fBpage-num-char\fR \fIc\fR" "req-np/imm"
.IP
Sets the page number character (used in titles) to
.I c .
.Ac "\fBpage-number\fR \fIN\fR" "req-np/imm"
.IP
Sets the current page number and the
.B %
register to
.I N .
Writes no output.
.Ac "\fBparse-absrel-num\fR \fIc N\fR" req-p
.IP
Parses a number or numeric expression from the request line, which may
be absolute, or relative
to
.I N .
.I c
is the default scaling unit to be used in evaluating expressions.
.Ac "\fBparse-char\fR" req-p
.IP
Parses a single character from the request line.
.Ac "\fBparse-condition\fR \fIc\fR" req-p
.IP
Parses a condition from the request line, tests it, and executes or skips
the rest of the line depending on the result of the test.
.I c
should be ``y'' if a following else-clause should be expected later (as for
.Rq ie )
or ``n'' if it should not be (as for
.Rq if ).
.Ac "\fBparse-filename\fR" req-p
.IP
Parses a filename from the request line.
.Ac "\fBparse-macro-args\fR" req-p
.IP
Parses all remaining arguments (in copy mode) to the
end of the request line.
Arguments are separated by spaces, or may be double-quoted to include
whitespace in an argument.
Note that a numeric argument is not evaluated in the same manner
as for the number-parsing actions.
To get around this, you can later interpolate the argument into a
string that's processed with
.B push-string:
.IP
.DS
# suppose first argument, if present, is numeric expression to set indent.
req XX parse-macro-args eol push-string ".if $$>0 .in $1\en"
.DE
.Ac "\fBparse-name\fR" req-p
.IP
Parses a name of the form
.I x
or
.I xx
from the request line.
This is used to pick up single string, macro, register, etc. names.
.Ac "\fBparse-num\fR \fIc\fR" req-p
.IP
Parses a number or numeric expression from the request line.
.I c
is the default scaling unit to be used in evaluating expressions.
.Ac "\fBparse-removelist\fR \fIc\fR" req-p
.IP
Parses a list of name of the form
.I x
or
.I xx
from the request line and removes them.
.I c
is ``y'' if the names refer to number registers, ``n'' if the names
refer to request, string, macro names.
.Ac "\fBparse-s\fR" req-p
.IP
An ugly hack used for the
.Rq bd
request. 
Parses an ``S'' from the request line if it is present, otherwise
puts an empty argument into the request argument vector.
.Ac "\fBparse-string-value\fR \fIc\fR" req-p
.IP
Parses a string in copy mode to the end of the input
line.
.I c
should be ``y'' if a leading double quote should be stripped off (as for
.Rq ds )
or ``n'' if it should not be (as for
.Rq tm ).
.Ac "\fBparse-tab-stops\fR" req-p
.IP
Parses the rest of the request line for tab stop settings.
If there are none,
.Cl reset-tabs
is written, which should be taken as meaning reset tab stops to ``every
half inch''.
(The
.I troff
manual doesn't specify how many stops are allowed; presumably postprocessors
should maintain ``several''.)
.IP
If explicit stops are given, the first one is written out as
.Cl first-tab
.I N
.I c ,
and any following are written as
.Cl next-tab
.I N
.I c .
When
.Cl first-tab
is seen, the postprocessor should clear any stops and begin collecting
new ones.
Each
.Cl next-tab
setting should be added to the current list.
.I N
is specified in basic units.
.I c
is
.B l
(left),
.B c
(center) or
.B r
(right).
.Ac "\fBparse-title\fR" req-p
.IP
Parses a title of the form \fB\'\fIleft\fB\'\fIcenter\fB\'\fIright\fB\'\fR
from the request line into
.I three
elements of
.I rargv [\|].
.Ac "\fBparse-transliteration\fR" req-p
.IP
Parses a transliteration list from the request line.
.Ac "\fBpoint-size\fR \fIN\fR" "req-np/imm"
.IP
Sets the point size to
.I N .
.I N
is number of points (unscaled), not basic units.
.Ac "\fBprocess-condition\fR" req-p
.IP
Executes or skips conditional input.
This is used for the
.Rq el
request, and tests whether the preceding matching
.Rq ie
succeeded or failed.
.Ac "\fBpush-file\fR \fIname\fR" req-np
.IP
Pushes input to the given file.
.Ac "\fBpush-string\fR \fIstring\fR" req-np
.IP
Pushes input to the given (anonymous) string.
To push to a named string or a request or macro, use
.B push-string
"\e*\fIx\fR"
(or "\e*(\fIxx\fR") or
.B push-string
"\fB.\fIxx\fR".
.Ac "\fBregister-format\fR \fIxx format\fR" "req-np/imm"
.IP
Assigns the given format to number register
.I xx .
.Ac "\fBrename\fR \fIxx yy\fR" "req-np/imm"
.IP
Renames request, macro, or string
.I xx
to
.I yy .
.Ac "\fBset-control\fR \fIc\fR" "req-np/imm"
.IP
Sets control character to
.I c
.B \. "" (
if
.I c
is missing).
.Ac "\fBset-control2\fR \fIc\fR" "req-np/imm"
.IP
Sets no-break control character to
.I c
.B \' "" (
if
.I c
is missing).
.Ac "\fBset-escape\fR \fIc\fR" "req-np/imm"
.IP
Restores escape processing, setting escape character to
.I c
(\fB\e\fR if
.I c
is missing).
.I c
cannot be an escaped or
.Es ( \fIxx\fR
special character, since then it would be recursive.
.Ac "\fBset-field\fR \fIdelim pad\fR" "req-np/imm"
.IP
Sets field delimiter character to
.I delim
and field padding character to
.I pad .
If
.I pad
is missing, the pad character becomes space.
If
.I delim
and
.I pad
are both missing, the field mechanism is turned off.
.Ac "\fBset-leader\fR \fIc\fR" "req-np/imm"
.IP
Sets leader repetition character to
.I c
or remove it if
.I c
is missing.
.Ac "\fBset-tab\fR \fIc\fR" "req-np, imm"
.IP
Sets tab repetition character to
.I c
or remove it if
.I c
is missing.
.Ac "\fBspace\fR \fIN\fR" "req-np/imm"
.IP
Spaces vertically by
.I N
units (negative = upward).
.Ac "\fBspace-size\fR \fIN\fR" "req-np/imm"
.IP
Sets space size to
.I N /36\fBm\fR.
.I N
is unscaled, not in basic units.
.Ac "\fBspacing\fR \fIN\fR" "req-np/imm"
.IP
Sets vertical base-line spacing to
.I N
units.
1
.B v
then becomes
.I N
units.
.Ac "\fBspecial-char\fR \fIxx output\fR" "req-np/imm"
.IP
Defines a special character with the given name and output sequence.
The name should be two characters.
The output sequence is the string that should be written when the
character is referenced.
.B special-char
is used in
.B imm
lines in action files to define the set of special characters that
.I troffcvt
should recognize.
This set should reflect the characters known by local versions of
.I troff .
.Ac "\fBswitch-file\fR \fIname\fR" req-np
.IP
Switches the current input file to
.I name .
.Ac "\fBtemp-indent\fR \fIN\fR" "req-np/imm"
.IP
Sets the temporary indent to
.I N .
.Ac "\fBtitle\fR" "req-np/imm"
.IP
Umm...
.Ac "\fBtitle-length\fR \fIN\fR" "req-np/imm"
.IP
Set the title length to
.I N .
.Ac "\fBtransliterate\fR \fIlist\fR" "req-np/imm"
.IP
.I list
should contain pairs of characters.
The first element of each pair will be transliterated into the second
element on output.
If the list has an odd number of characters, the last is transliterated
to a space.
.Ac "\fBunderline\fR \fIN\fR" "req-np/imm"
.IP
Turns on underlining and writes
.Cl underline
to the output.
Underlining is turned off after
.I N
lines
of text input
(1 line if
.I N
is empty).
.Ac "\fBunderline-font\fR \fIF\fR" "req-np/imm"
.IP
Sets the underline font to
.I F .
.NH
Creating a New Action
.LP
If the set of actions is insufficient for your
purposes, here's how to modify
.I troffcvt
to write a new one.
.IP (i)
Read the rest of this section, then study some of the existing
actions in
.I troffcvt
source code to see how they are implemented.
.IP (ii)
Pick a name for the action.
This will be the name used in action files to refer to it.
.IP (iii)
Decide whether the function used to implement the action should take
any arguments, that is, whether the action should be followed by any
arguments when used in an action file.
You must also decide whether it should be allowed on
.B imm
lines and/or
.B req
lines (and if allowed on the latter, whether allowed in the non-parsing
or parsing sections).
.IP (iv)
Write the action function.
It should be declared like this:
.IP
.DS
.ta .75i
int ActionFunction (argc, argv)
int	argc;
UChar	**argv;
{
}
.DE
.IP
The function should perform the action and return non-zero for success,
zero otherwise.
Possibly it should simply panic if something catastrophic (unrecoverable)
happens.
Note that sometimes it is useful to return non-zero for certain kinds
of failures.
If an action in an action list returns zero,
none of the following actions in the list are executed.
You may want them to execute anyway; if so, the function shouldn't
return zero.
.IP
.I argc
is the number of action arguments and
.I argv
is an array of pointers to them.
(There is a NULL pointer following the last argument, i.e.,
\fIargv\fR[\fIargc\fR]
is NULL.)
.IP
The function should make no direct reference to the request arguments
.I rargc
and
.I rargv
unless it is an argument parsing action.
If your action can be used on
.B imm
lines, 
there won't be any request arguments available.
It's also a good idea for the function to refrain from modifying the arguments
passed to it.
(Better to make copies first and modify the copies.)
.IP (v)
List the action in the
.I aiTab [\|]
array in
.I action.c .
An extern for the function should also go into
.I troffcvt.h .
If the action parses arguments, it goes in
.I parse.c ,
otherwise
.I req.c .
